@@ -15,6 +15,7 @@ class TestParser(unittest.TestCase):
         ignore_override(True)
 
     def test_expression_1(self):
+        delete_namespace()
         expr = r'-(\frac{2}{3} + 2\sqrt[5]{x + 3})'
         self.assertEqual(
             str(parse_expr(expr)),
@@ -22,14 +23,16 @@ class TestParser(unittest.TestCase):
         )
 
     def test_expression_2(self):
-        expr = r'e^{{\ln x}} + \sin(\sin^{-1} y) - \tanh(xy)'
+        delete_namespace()
+        expr = r'e^{\ln x} + \sin(\sin^{-1} y) - \tanh(xy)'
         self.assertEqual(
             str(parse_expr(expr)),
             'x + y - tanh(x*y)'
         )
 
     def test_expression_3(self):
-        expr = r'\partial_x (x^{{2}} + 2x)'
+        delete_namespace()
+        expr = r'\partial_x (x^2 + 2x)'
         self.assertEqual(
             str(parse_expr(expr).doit()),
             '2*x + 2'
@@ -39,22 +42,22 @@ class TestParser(unittest.TestCase):
         function = Function('Tensor')(Symbol('T'))
         self.assertEqual(
             Parser._generate_covdrv(function, 'beta'),
-            r'\nabla_\beta T = \partial_\beta T'
+            r'\nabla_{\beta} T = \partial_{\beta} T'
         )
         function = Function('Tensor')(Symbol('TUU'), Symbol('mu'), Symbol('nu'))
         self.assertEqual(
             Parser._generate_covdrv(function, 'beta'),
-            r'\nabla_\beta T^{\mu \nu} = \partial_\beta T^{\mu \nu} + \text{Gamma}^\mu_{a \beta} (T^{a \nu}) + \text{Gamma}^\nu_{a \beta} (T^{\mu a})'
+            r'\nabla_{\beta} T^{\mu \nu} = \partial_{\beta} T^{\mu \nu} + \text{Gamma}^{\mu}_{a \beta} (T^{a \nu}) + \text{Gamma}^{\nu}_{a \beta} (T^{\mu a})'
         )
         function = Function('Tensor')(Symbol('TUD'), Symbol('mu'), Symbol('nu'))
         self.assertEqual(
             Parser._generate_covdrv(function, 'beta'),
-            r'\nabla_\beta T^\mu_\nu = \partial_\beta T^\mu_\nu + \text{Gamma}^\mu_{a \beta} (T^a_\nu) - \text{Gamma}^a_{\nu \beta} (T^\mu_a)'
+            r'\nabla_{\beta} T^{\mu}_{\nu} = \partial_{\beta} T^{\mu}_{\nu} + \text{Gamma}^{\mu}_{a \beta} (T^{a}_{\nu}) - \text{Gamma}^{a}_{\nu \beta} (T^{\mu}_{a})'
         )
         function = Function('Tensor')(Symbol('TDD'), Symbol('mu'), Symbol('nu'))
         self.assertEqual(
             Parser._generate_covdrv(function, 'beta'),
-            r'\nabla_\beta T_{\mu \nu} = \partial_\beta T_{\mu \nu} - \text{Gamma}^a_{\mu \beta} (T_{a \nu}) - \text{Gamma}^a_{\nu \beta} (T_{\mu a})'
+            r'\nabla_{\beta} T_{\mu \nu} = \partial_{\beta} T_{\mu \nu} - \text{Gamma}^{a}_{\mu \beta} (T_{a \nu}) - \text{Gamma}^{a}_{\nu \beta} (T_{\mu a})'
         )
 
     def test_expression_5(self):
@@ -67,29 +70,29 @@ class TestParser(unittest.TestCase):
         function = Function('Tensor')(Symbol('vU_cdD'), Symbol('mu'), Symbol('b'))
         self.assertEqual(
             Parser._generate_covdrv(function, 'a'),
-            r'\nabla_a \nabla_b v^\mu = \partial_a \nabla_b v^\mu + \text{Gamma}^\mu_{c a} (\nabla_b v^c) - \text{Gamma}^c_{b a} (\nabla_c v^\mu)'
+            r'\nabla_{a} \nabla_{b} v^{\mu} = \partial_{a} \nabla_{b} v^{\mu} + \text{Gamma}^{\mu}_{c a} (\nabla_{b} v^{c}) - \text{Gamma}^{c}_{b a} (\nabla_{c} v^{\mu})'
         )
 
     def test_expression_6(self):
         function = Function('Tensor')(Symbol('g'))
         self.assertEqual(
             Parser._generate_liedrv(function, 'beta', 2),
-            r'\mathcal{L}_\text{beta} g = \text{beta}^a \partial_a g + (2)(\partial_a \text{beta}^a) g'
+            r'\mathcal{L}_\text{beta} g = \text{beta}^{a} \partial_{a} g + (2)(\partial_{a} \text{beta}^{a}) g'
         )
         function = Function('Tensor')(Symbol('gUU'), Symbol('i'), Symbol('j'))
         self.assertEqual(
             Parser._generate_liedrv(function, 'beta'),
-            r'\mathcal{L}_\text{beta} g^{i j} = \text{beta}^a \partial_a g^{i j} - (\partial_a \text{beta}^i) g^{a j} - (\partial_a \text{beta}^j) g^{i a}'
+            r'\mathcal{L}_\text{beta} g^{i j} = \text{beta}^{a} \partial_{a} g^{i j} - (\partial_{a} \text{beta}^{i}) g^{a j} - (\partial_{a} \text{beta}^{j}) g^{i a}'
         )
         function = Function('Tensor')(Symbol('gUD'), Symbol('i'), Symbol('j'))
         self.assertEqual(
             Parser._generate_liedrv(function, 'beta'),
-            r'\mathcal{L}_\text{beta} g^i_j = \text{beta}^a \partial_a g^i_j - (\partial_a \text{beta}^i) g^a_j + (\partial_j \text{beta}^a) g^i_a'
+            r'\mathcal{L}_\text{beta} g^{i}_{j} = \text{beta}^{a} \partial_{a} g^{i}_{j} - (\partial_{a} \text{beta}^{i}) g^{a}_{j} + (\partial_{j} \text{beta}^{a}) g^{i}_{a}'
         )
         function = Function('Tensor')(Symbol('gDD'), Symbol('i'), Symbol('j'))
         self.assertEqual(
             Parser._generate_liedrv(function, 'beta'),
-            r'\mathcal{L}_\text{beta} g_{i j} = \text{beta}^a \partial_a g_{i j} + (\partial_i \text{beta}^a) g_{a j} + (\partial_j \text{beta}^a) g_{i a}'
+            r'\mathcal{L}_\text{beta} g_{i j} = \text{beta}^{a} \partial_{a} g_{i j} + (\partial_{i} \text{beta}^{a}) g_{a j} + (\partial_{j} \text{beta}^{a}) g_{i a}'
         )
 
     def test_srepl_macro(self):
@@ -162,16 +165,16 @@ class TestParser(unittest.TestCase):
                 % keydef basis [x, y]
                 % vardef 'uD' (2D), 'wD' (2D)
                 % keydef index [a-z] (2D)
-                u_x = x^{{2}} + 2x \\
+                u_x = x^2 + 2x \\
                 u_y = y\sqrt{x} \\
                 v_a = u_a + w_a \\
                 % assign -diff_type=dD 'wD', 'vD'
-                T_{ab} = \partial^2_x v_x (\partial_b v_a)
+                T_{ab} = \partial_b v_a
             """)),
             {'uD', 'wD', 'vD', 'vD_dD', 'wD_dD', 'TDD'}
         )
         self.assertEqual(str(TDD),
-            '[[2*wD_dD00 + 4*x + 4, 2*wD_dD01], [2*wD_dD10 + y/sqrt(x), 2*wD_dD11 + 2*sqrt(x)]]'
+            '[[wD_dD00 + 2*x + 2, wD_dD01], [wD_dD10 + y/(2*sqrt(x)), wD_dD11 + sqrt(x)]]'
         )
 
     def test_assignment_5(self):
@@ -182,15 +185,15 @@ class TestParser(unittest.TestCase):
                 % vardef 'uD' (2D), 'wD' (2D)
                 % assign -diff_type=symbolic 'uD'
                 % keydef index [a-z] (2D)
-                u_x = x^{{2}} + 2x \\
+                u_x = x^2 + 2x \\
                 u_y = y\sqrt{x} \\
                 v_a = u_a + w_a \\
-                T_{bc} = \partial^2_x v_x (\vphantom{dD} \partial_c v_b)
+                T_{bc} = \vphantom{dD} \partial_c v_b
             """)),
             {'uD', 'wD', 'vD', 'vD_dD', 'wD_dD', 'TDD'}
         )
         self.assertEqual(str(TDD),
-            '[[2*wD_dD00 + 4*x + 4, 2*wD_dD01], [2*wD_dD10 + y/sqrt(x), 2*wD_dD11 + 2*sqrt(x)]]'
+            '[[wD_dD00 + 2*x + 2, wD_dD01], [wD_dD10 + y/(2*sqrt(x)), wD_dD11 + sqrt(x)]]'
         )
 
     def test_assignment_6(self):
@@ -215,8 +218,8 @@ class TestParser(unittest.TestCase):
             % vardef -const 'r'
             % keydef index [a-z] (2D)
             \begin{align*}
-                g_{0 0} &= r^{{2}} \\
-                g_{1 1} &= r^{{2}} \sin^2(\theta)
+                g_{0 0} &= r^2 \\
+                g_{1 1} &= r^2 \sin^2(\theta)
             \end{align*}
             % assign -metric 'gDD'
             \begin{align*}
@@ -251,10 +254,10 @@ class TestParser(unittest.TestCase):
         delete_namespace()
         self.assertEqual(
             set(parse(r"""
-                % vardef -metric 'gDD' (4D)
+                % vardef -symmetry=sym01 'gDD' (4D)
                 \gamma_{ij} = g_{ij}
             """)),
-            {'gUU', 'gdet', 'epsilonUUUU', 'gDD', 'gammaDD'}
+            {'gDD', 'gammaDD'}
         )
         self.assertEqual(str(gammaDD),
             '[[gDD11, gDD12, gDD13], [gDD12, gDD22, gDD23], [gDD13, gDD23, gDD33]]'
@@ -283,7 +286,35 @@ class TestParser(unittest.TestCase):
                 % vardef 'ADDD', 'AUUU'
                 B^{a b}_c = A^{a b}_c
             """)),
-            {'ADDD', 'BUUD', 'gdet', 'gDD', 'AUUD', 'AUUU', 'gUU', 'epsilonUUU'}
+            {'gDD', 'epsilonUUU', 'gdet', 'gUU', 'GammaUDD', 'ADDD', 'AUUU', 'AUUD', 'BUUD'}
+        )
+
+    def test_assignment_11(self):
+        delete_namespace()
+        self.assertEqual(
+            set(parse(r"""
+                % vardef 'vD'
+                w = v_{x_2}
+            """)),
+            {'vD', 'w'}
+        )
+        self.assertEqual(str(w),
+            'vD2'
+        )
+
+    def test_assignment_12(self):
+        delete_namespace()
+        self.assertEqual(
+            set(parse(r"""
+                % keydef basis [x, y, z]
+                % vardef -zero 'vD'
+                v_z = y^2 + 2y \\
+                w = v_{x_2}
+            """)),
+            {'vD', 'w'}
+        )
+        self.assertEqual(str(w),
+            'y**2 + 2*y'
         )
 
     def test_example_1(self):
@@ -307,7 +338,7 @@ class TestParser(unittest.TestCase):
                 % vardef 'vD' (3D)
                 v^\mu = g^{\mu\nu} v_\nu
             """)),
-            {'gDD', 'gdet', 'epsilonDDD', 'gUU', 'vD', 'vU'}
+            {'gUU', 'epsilonDDD', 'gdet', 'gDD', 'GammaUDD', 'vD', 'vU'}
         )
         self.assertEqual(str(vU),
             '[gUU00*vD0 + gUU01*vD1 + gUU02*vD2, gUU01*vD0 + gUU11*vD1 + gUU12*vD2, gUU02*vD0 + gUU12*vD1 + gUU22*vD2]'
@@ -367,8 +398,8 @@ class TestParser(unittest.TestCase):
             \begin{align}
                 g_{t t} &= -\left(1 - \frac{2GM}{r}\right) \\
                 g_{r r} &=  \left(1 - \frac{2GM}{r}\right)^{-1} \\
-                g_{\theta \theta} &= r^{{2}} \\
-                g_{\phi \phi} &= r^{{2}} \sin^2\theta
+                g_{\theta \theta} &= r^2 \\
+                g_{\phi \phi} &= r^2 \sin^2\theta
             \end{align}
             % assign -metric 'gDD'
         """)
